@@ -8,7 +8,7 @@
 
 use std::{collections::HashMap, str::Utf8Error};
 
-use ops::{Dim, Id, Op};
+use ops::{Dim, Op};
 use thiserror::Error;
 
 mod ops;
@@ -32,27 +32,12 @@ pub type SpirvResult<T> = ::std::result::Result<T, Error>;
 /// Stores the reflection info of a single SPIRV module.
 #[derive(Debug)]
 pub struct Module {
+    /// Stores information about all type declarations that exist in the given SPIRV module.
     types: HashMap<u32, Type>,
-    entry_points: Vec<EntryPoint>,
-}
-
-/// Describes a single `EntryPoint` in a SPIR-V module.
-///
-/// A SPIR-V module can have multiple entry points with different names, each defining a single shader.
-#[derive(Debug)]
-pub struct EntryPoint {
-    /// The name of the entry point, used for identification
-    pub name: String,
-    /// The [`ExecutionModel`] of the entry point, selects which type of shader this entry point defines
-    pub execution_model: ExecutionModel,
-    /// All uniform variables used in this shader
-    pub uniforms: Vec<UniformVariable>,
-    /// All push constant variables used in this shader
-    pub push_constants: Vec<PushConstantVariable>,
-    /// All inputs used in this shader
-    pub inputs: Vec<LocationVariable>,
-    /// All outputs used in this shader
-    pub outputs: Vec<LocationVariable>,
+    /// Stores information about all uniform variables that exist in the given SPIRV module.
+    uniforms: Vec<Variable>,
+    /// Stores information about all push constant variables that exist in the given SPIRV module.
+    push_constants: Vec<PushConstantVariable>,
 }
 
 impl Module {
@@ -80,145 +65,59 @@ impl Module {
             ops.push(op);
         }
 
-        // All OpConstant values are stored in this Map
+        // All OpConstant values are stored in this Vec
         let mut constants = HashMap::new();
-        // All type declarations are stored in this Map
+        // All type declarations are stored in this Vec
         let mut types = HashMap::new();
-        // All variable declarations are stored in this Map
+        // All variable declarations are stored in this Vec
         let mut vars = HashMap::new();
-        // All entry points declarations are stored in this Vec
-        let mut entries = Vec::new();
 
-        Self::collect_types_and_vars(&ops, &mut types, &mut constants, &mut vars, &mut entries)?;
+        Self::collect_types_and_vars(&ops, &mut types, &mut constants, &mut vars)?;
         Self::collect_decorations_and_names(&ops, &mut types, &mut vars);
 
         // uniforms are all variables that are a pointer with a storage class of Uniform or UniformConstant
-        let uniforms: HashMap<_, _> = vars
+        let uniforms = vars
             .iter()
-            .filter_map(|(id, var)| {
+            .filter_map(|(_id, var)| {
                 if let Some(Type::Pointer {
                     storage_class: StorageClass::Uniform | StorageClass::UniformConstant,
                     pointed_type_id,
                 }) = types.get(&var.type_id)
                 {
-                    Some((
-                        *id,
-                        UniformVariable {
-                            set: var.set?,
-                            binding: var.binding?,
-                            type_id: *pointed_type_id, // for convenience, we store the pointed-to type instead of the pointer, since every uniform is a pointer
-                            name: var.name.clone(),
-                        },
-                    ))
+                    Some(Variable {
+                        set: var.set,
+                        binding: var.binding,
+                        type_id: *pointed_type_id, // for convenience, we store the pointed-to type instead of the pointer, since every uniform is a pointer
+                        name: var.name.clone(),
+                    })
                 } else {
                     None
                 }
             })
             .collect();
 
-        let push_constants: HashMap<_, _> = vars
+        let push_constants = vars
             .iter()
-            .filter_map(|(id, var)| {
+            .filter_map(|(_id, var)| {
                 if let Some(Type::Pointer {
                     storage_class: StorageClass::PushConstant,
                     pointed_type_id,
                 }) = types.get(&var.type_id)
                 {
-                    Some((
-                        *id,
-                        PushConstantVariable {
-                            type_id: *pointed_type_id,
-                            name: var.name.clone(),
-                        },
-                    ))
+                    Some(PushConstantVariable {
+                        type_id: *pointed_type_id,
+                        name: var.name.clone(),
+                    })
                 } else {
                     None
                 }
             })
             .collect();
 
-        let inputs: HashMap<_, _> = vars
-            .iter()
-            .filter_map(|(id, var)| {
-                if let Some(Type::Pointer {
-                    storage_class: StorageClass::Input,
-                    pointed_type_id,
-                }) = types.get(&var.type_id)
-                {
-                    Some((
-                        *id,
-                        LocationVariable {
-                            location: var.location?,
-                            type_id: *pointed_type_id,
-                            name: var.name.clone(),
-                        },
-                    ))
-                } else {
-                    None
-                }
-            })
-            .collect();
-
-        let outputs: HashMap<_, _> = vars
-            .iter()
-            .filter_map(|(id, var)| {
-                if let Some(Type::Pointer {
-                    storage_class: StorageClass::Output,
-                    pointed_type_id,
-                }) = types.get(&var.type_id)
-                {
-                    Some((
-                        *id,
-                        LocationVariable {
-                            location: var.location?,
-                            type_id: *pointed_type_id,
-                            name: var.name.clone(),
-                        },
-                    ))
-                } else {
-                    None
-                }
-            })
-            .collect();
-
-        let entry_points = entries
-            .iter()
-            .map(|e| {
-                let uniforms = e
-                    .interface
-                    .iter()
-                    .filter_map(|id| uniforms.get(&id.0).cloned())
-                    .collect();
-                let push_constants = e
-                    .interface
-                    .iter()
-                    .filter_map(|id| push_constants.get(&id.0).cloned())
-                    .collect();
-                let inputs = e
-                    .interface
-                    .iter()
-                    .filter_map(|id| inputs.get(&id.0).cloned())
-                    .collect();
-                let outputs = e
-                    .interface
-                    .iter()
-                    .filter_map(|id| outputs.get(&id.0).cloned())
-                    .collect();
-
-                EntryPoint {
-                    name: e.name.clone(),
-                    execution_model: e.execution_model,
-                    uniforms,
-                    push_constants,
-                    inputs,
-                    outputs,
-                }
-            })
-            .collect();
-
         Ok(Self {
             types,
-            entry_points,
+            uniforms,
+            push_constants,
         })
     }
 
@@ -227,26 +126,36 @@ impl Module {
         self.types.get(&type_id)
     }
 
-    /// Returns the [`EntryPoint`] definitions contained in the given SPIR-V module
-    pub fn get_entry_points(&self) -> &[EntryPoint] {
-        &self.entry_points
+    /// Returns all uniform variables declared in the given SPIR-V module.
+    pub fn get_uniforms(&self) -> &[Variable] {
+        &self.uniforms
     }
 
-    fn get_type_size(&self, type_id: u32, stride: Option<u32>) -> Option<u32> {
+    /// Returns all push-constant variables declared in the given SPIR-V module.
+    pub fn get_push_constants(&self) -> &[PushConstantVariable] {
+        &self.push_constants
+    }
+
+    /// Calculates the size of a primitive type or Struct.
+    ///
+    /// # Returns
+    /// The size of the type indicated by `type_id`
+    /// or [`None`] if the size of the given type is not known.
+    pub fn get_type_size(&self, type_id: u32) -> Option<u32> {
         if let Some(ty) = self.types.get(&type_id) {
             match ty {
                 Type::Int32 | Type::UInt32 | Type::Float32 => Some(4),
                 Type::Vec2 => Some(8),
                 Type::Vec3 => Some(12),
                 Type::Vec4 => Some(16),
-                Type::Mat3 => stride.map(|stride| stride * 2 + 12), // two rows/columns + sizeof(Vec3)
-                Type::Mat4 => stride.map(|stride| stride * 3 + 16), // three rows/columns + sizeof(Vec4)
+                Type::Mat3 => Some(48), // Mat3 works like three Vec3 after another, Vec3 has alignment of Vec4
+                Type::Mat4 => Some(64),
                 Type::Struct { elements, .. } => {
                     // Since there is no Size Decoration in SPIRV that tells us the size,
                     // we calculate it from the offset of the last member and its size.
                     let last_element = elements.iter().max_by_key(|e| e.offset.unwrap_or(0))?;
                     let offset = last_element.offset?;
-                    let size = self.get_member_size(last_element)?;
+                    let size = self.get_type_size(last_element.type_id)?;
 
                     Some(offset + size)
                 }
@@ -257,21 +166,11 @@ impl Module {
         }
     }
 
-    /// Returns the size of a given [`StructMember`], if known.
-    pub fn get_member_size(&self, member: &StructMember) -> Option<u32> {
-        self.get_type_size(member.type_id, Some(member.stride))
-    }
-
-    /// Returns the size of a given [`UniformVariable`], [`PushConstantVariable`] or [`LocationVariable`], if known.
-    pub fn get_var_size<T: Variable>(&self, var: &T) -> Option<u32> {
-        self.get_type_size(var.get_type_id(), None)
-    }
-
     /// Parses all the Op*Decoration and Op*Name instructions
     fn collect_decorations_and_names(
         ops: &[Op],
         types: &mut HashMap<u32, Type>,
-        vars: &mut HashMap<u32, RawVariable>,
+        vars: &mut HashMap<u32, Variable>,
     ) {
         for op in ops {
             match op {
@@ -304,35 +203,16 @@ impl Module {
                             target.set = Some(*set);
                         }
                     }
-                    ops::Decoration::Location { loc } => {
-                        if let Some(target) = vars.get_mut(&target.0) {
-                            target.location = Some(*loc);
-                        }
-                    }
                     _ => {}
                 },
                 Op::OpMemberDecorate {
                     target,
                     member_index,
-                    decoration,
+                    decoration: ops::Decoration::Offset { offset },
                 } => {
                     if let Some(Type::Struct { elements, .. }) = types.get_mut(&target.0) {
                         if elements.len() > *member_index as usize {
-                            match decoration {
-                                ops::Decoration::RowMajor {} => {
-                                    elements[*member_index as usize].row_major = true;
-                                }
-                                ops::Decoration::ColMajor {} => {
-                                    elements[*member_index as usize].row_major = false;
-                                }
-                                ops::Decoration::MatrixStride { stride } => {
-                                    elements[*member_index as usize].stride = *stride;
-                                }
-                                ops::Decoration::Offset { offset } => {
-                                    elements[*member_index as usize].offset = Some(*offset);
-                                }
-                                _ => {}
-                            }
+                            elements[*member_index as usize].offset = Some(*offset);
                         }
                     }
                 }
@@ -346,8 +226,7 @@ impl Module {
         ops: &[Op],
         types: &mut HashMap<u32, Type>,
         constants: &mut HashMap<u32, u32>,
-        vars: &mut HashMap<u32, RawVariable>,
-        entries: &mut Vec<RawEntryPoint>,
+        vars: &mut HashMap<u32, Variable>,
     ) -> SpirvResult<()> {
         for op in ops {
             match op {
@@ -502,8 +381,6 @@ impl Module {
                                     name: None,
                                     type_id: e.0,
                                     offset: None,
-                                    row_major: true,
-                                    stride: 16,
                                 })
                                 .collect(),
                         },
@@ -522,8 +399,6 @@ impl Module {
                                 ops::StorageClass::UniformConstant {}
                                 | ops::StorageClass::Uniform {} => StorageClass::Uniform,
                                 ops::StorageClass::PushConstant {} => StorageClass::PushConstant,
-                                ops::StorageClass::Input {} => StorageClass::Input,
-                                ops::StorageClass::Output {} => StorageClass::Output,
                             },
                             pointed_type_id: pointed_type.0,
                         },
@@ -548,35 +423,14 @@ impl Module {
                 } => {
                     vars.insert(
                         result.0,
-                        RawVariable {
+                        Variable {
                             set: None,
                             binding: None,
-                            location: None,
                             type_id: result_type.0,
                             name: None,
                         },
                     );
                 }
-                Op::OpEntryPoint {
-                    execution_model,
-                    func: _,
-                    name,
-                    interface,
-                } => {
-                    entries.push(RawEntryPoint {
-                        name: name.clone(),
-                        execution_model: match execution_model {
-                            ops::ExecutionModel::Unknown => {
-                                return Err(Error::Other(
-                                    "Unknown execution model in entry point".to_string(),
-                                ))
-                            }
-                            ops::ExecutionModel::Vertex {} => ExecutionModel::Vertex,
-                            ops::ExecutionModel::Fragment {} => ExecutionModel::Fragment,
-                        },
-                        interface: interface.clone(),
-                    });
-                }
                 _ => {}
             }
         }
@@ -654,16 +508,9 @@ pub enum Type {
 /// Describes a single member of a [`Type::Struct`] type
 #[derive(Debug)]
 pub struct StructMember {
-    /// The name of the member variable (if known)
     pub name: Option<String>,
-    /// The type id of the member's [`Type`]
     pub type_id: u32,
-    /// The offset within the struct of this member (if known)
     pub offset: Option<u32>,
-    /// For matrix members: whether this matrix is stored in row major order
-    pub row_major: bool,
-    /// For matrix members: The stride between rows/columns of the matrix
-    pub stride: u32,
 }
 
 /// Describes what type of storage a pointer points to
@@ -677,92 +524,23 @@ pub enum StorageClass {
     UniformConstant,
     /// The pointer is a push constant
     PushConstant,
-    /// The pointer is an input variable
-    Input,
-    /// The pointer is an output variable
-    Output,
-}
-
-/// The execution model of an [`EntryPoint`].
-#[derive(Debug, Clone, Copy)]
-#[non_exhaustive]
-pub enum ExecutionModel {
-    /// A Vertex Shader
-    Vertex,
-    /// A Fragment Shader
-    Fragment,
 }
 
+/// Describes a variable declared in a SPIRV module
 #[derive(Debug, Clone)]
-struct RawVariable {
-    set: Option<u32>,
-    binding: Option<u32>,
-    location: Option<u32>,
-    type_id: u32,
-    name: Option<String>,
-}
-
-#[derive(Debug)]
-struct RawEntryPoint {
-    name: String,
-    execution_model: ExecutionModel,
-    interface: Vec<Id>,
-}
-
-/// Describes a uniform variable declared in a SPIRV module
-#[derive(Debug, Clone)]
-pub struct UniformVariable {
+pub struct Variable {
     /// Which DescriptorSet the variable is contained in (if known)
-    pub set: u32,
+    pub set: Option<u32>,
     /// Which DescriptorSet binding the variable is contained in (if known)
-    pub binding: u32,
+    pub binding: Option<u32>,
     /// The type id of the variable's [`Type`]
     pub type_id: u32,
     /// The variables name (if known)
     pub name: Option<String>,
 }
 
-/// Describes a push constant variable declared in a SPIRV module
 #[derive(Debug, Clone)]
 pub struct PushConstantVariable {
-    /// The type id of the variable's [`Type`]
     pub type_id: u32,
-    /// The variables name (if known)
     pub name: Option<String>,
 }
-
-/// Describes an input or output variable declared in a SPIRV module
-#[derive(Debug, Clone)]
-pub struct LocationVariable {
-    /// The location of the variable (e.g. GLSL `layout(location=XXX)`)
-    pub location: u32,
-    /// The type id of the variable's [`Type`]
-    pub type_id: u32,
-    /// The variable's name (if known)
-    pub name: Option<String>,
-}
-
-mod private {
-    pub trait Variable {
-        fn get_type_id(&self) -> u32;
-    }
-}
-
-pub trait Variable: private::Variable {}
-impl<T: private::Variable> Variable for T {}
-
-impl private::Variable for UniformVariable {
-    fn get_type_id(&self) -> u32 {
-        self.type_id
-    }
-}
-impl private::Variable for PushConstantVariable {
-    fn get_type_id(&self) -> u32 {
-        self.type_id
-    }
-}
-impl private::Variable for LocationVariable {
-    fn get_type_id(&self) -> u32 {
-        self.type_id
-    }
-}
